import logging
import sys


class DefaultLogger:
    DEFAULT_LOG_FORMAT = '[%(levelname)-7s] - %(asctime)s - %(module)s.%(funcName)s:%(lineno)d: %(message)s'
    DEFAULT_DATE_FORMAT = '%Y-%m-%d %H:%M:%S'

    @classmethod
    def create_logger(cls, appName: str, logLevel=logging.DEBUG, logFormat=None, dateFormat=None) -> logging.Logger:
        logger = logging.getLogger(appName)
        return cls.configure_logger(logger, logLevel, logFormat, dateFormat)

    @classmethod
    def configure_logger(cls, logger: logging.Logger,
                         logLevel=logging.DEBUG,
                         logFormat=None,
                         dateFormat=None) -> logging.Logger:
        logFormatter = cls.__create_formatter(logFormat, dateFormat)

        logger.setLevel(logLevel)

        outHandler = logging.StreamHandler(sys.stdout)
        outHandler.setFormatter(logFormatter)
        outHandler.setLevel(logging.DEBUG)
        outHandler.addFilter(lambda record: record.levelno <= logging.INFO)
        logger.addHandler(outHandler)

        errHandler = logging.StreamHandler(sys.stderr)
        errHandler.setFormatter(logFormatter)
        errHandler.setLevel(logging.WARNING)
        logger.addHandler(errHandler)
        return logger

    @classmethod
    def __create_formatter(cls, logFormat=None, dateFormat=None) -> logging.Formatter:
        if logFormat is None:
            logFormat = cls.DEFAULT_LOG_FORMAT
        if dateFormat is None:
            dateFormat = cls.DEFAULT_DATE_FORMAT

        return logging.Formatter(fmt=logFormat, datefmt=dateFormat)
